import 'dart:io';
import 'package:comet_cli/config.dart';
import 'package:comet_cli/utils/progress_star.dart';
import 'package:comet_cli/functions/read_yaml.dart';
import 'package:comet_cli/functions/resolve_src_shelves.dart';
import 'package:comet_cli/functions/run_flutter_run.dart';
import 'package:comet_cli/functions/watch_to_restart.dart';

Future<void> start() async {
  // Cometプロジェクトかどうか
  final cometYaml = readCometYaml();
  if (cometYaml == null) {
    print('Not found $cometYamlName');
    return;
  }

  ProgressStar.start('building..');

  await Process.run(
    'comet',
    [
      'build',
    ],
    runInShell: true,
  );

  ProgressStar.message('flutter run..');

  await runFlutterRun(
    cometYaml.flutterProject.root,
    (process) async {
      // on process started
      final foldersToWatch = await resolveSrcShelves(cometYaml);
      watchToRestart(foldersToWatch, process);
    },
    () {
      // maybe app started
      ProgressStar.blink('watching .md..');
    },
  );

  // on app finished
  ProgressStar.stop();
  exit(0);
}
