import 'dart:io';
import 'package:comet_cli/functions/run_flutter.dart';
import 'package:comet_cli/functions/run_flutter_restart.dart';
import 'package:watcher/watcher.dart';

// - onMaybeLaunched: アプリ起動の目安 将来のFlutterの変更で壊れるかもしれない
Future<void> runFlutterRun(
  String project,
  void Function(Process process) onProcessStarted,
  void Function() onMaybeLaunched,
) async {
  final process = await runFlutter(
    project,
    const [
      'run',
      '-d',
      'chrome',
    ],
    onStdout: (msg) {
      // アプリ起動を判断するためのキーワード
      const appLaunchedKeyword = 'The Flutter DevTools';
      if (msg.startsWith(appLaunchedKeyword)) {
        onMaybeLaunched();
      }
      stdout.write(msg);
    },
    onStderr: (msg) {
      stdout.write(msg);
    },
  );

  onProcessStarted(process);

  const watcherDelay = Duration(seconds: 1);
  final watcher = DirectoryWatcher(
    '$project${Platform.pathSeparator}lib',
    pollingDelay: watcherDelay,
  );
  watcher.events.listen((change) async {
    if (change.type == ChangeType.MODIFY) {
      await runFlutterRestart(process);
    }
  });

  await process.exitCode;
}
